/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;



public abstract class Content implements Serializable{
	private static final long serialVersionUID = 1L;
	
	// URI for the content
	public URI uri;

	public Content(URI uri){
		this.uri = uri;
	}
	
	@Override
	public String toString(){
		return uri.toString();
	}
	
	public String id(){
		return uri.id();
	}
	
	public abstract void store(FileAdapter fa);
	public abstract void play(GuiAdapter gui);
}

class FileContent extends Content{
	private static final long serialVersionUID = 1L;
	
	// Separate file data from the FileShape
	public static final long MAX_FILE_SIZE = 1024L*1024L*1L; //about 1M
	private static final int BUFSIZE = 1024;	// 1k
	
	public String relpath;
	public byte[] data;
	
	public FileContent(FileURI uri, String abspath){
		super(uri);
		
		this.uri = uri;
		try {
			File f = new File(abspath);
			this.relpath = f.getName();
			
			if(!initialize(f))
				System.err.println("FileContent::Failed to generate File Content!!! "+ abspath);
		} 
		catch (FileNotFoundException e) {
			System.err.println("FileContent::There is no file, "+relpath);
			e.printStackTrace();
		} 
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static FileContent fileContentFactory(FileURI uri, String abspath){
		if(uri.size > MAX_FILE_SIZE)
			return null;
		return new FileContent(uri, abspath);
	}
	
	private boolean initialize(File file) throws IOException, FileNotFoundException{
		if(!file.isFile())
			return false;
		
		FileInputStream fis = new FileInputStream(file);
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		
		byte[] buf = new byte[BUFSIZE];
		int read;
		
		while(-1 != (read=fis.read(buf)))
			bos.write(buf, 0, read);
		
		bos.flush();
		data = bos.toByteArray();
		
		if(data.length != ((FileURI)uri).size)
			new Exception("File size is not matched! "+"bytearray.length="+data.length+"  file.length()="+((FileURI)uri).size);
		return true;
	}
	
	@Override
	public void play(GuiAdapter gui){
		//System.out.println("Play");
	}
	
	@Override
	public void store(FileAdapter fa){
		fa.store((FileURI)uri, relpath, data);
	}
}
