/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;


public abstract class ContentShape extends Shape implements Cloneable{
	public URI uri;
	
	public ContentShape(URI uri, Color color, Point2df[] points, int len){
		super(uri.id(), color, points, len);		
		this.uri = uri;
	}
	
	@Override
	public String toString(){
		return "ContentShape-"+id;
	}
	
	@Override
	public ContentShape clone(){
		return (ContentShape)super.clone();
	}
}

class FileContentShape extends ContentShape implements Cloneable{
	private static final long serialVersionUID = 1L;
	
	public static final int IMAGE_SIZE = 50;
	public static final int TEXT_SIZE = 10;
	public static final int WIDTH = 50;
	public static final int HEIGHT = IMAGE_SIZE+TEXT_SIZE;
	public static final int POINT_INTERVAL = 25;
	
	Point2df center;
	String text;
	
	public FileContentShape(FileURI uri, Color color, Point2df[] center){
		super(uri, color, center, 1);
		this.center = new Point2df(center[0].x, center[0].y);
		this.text = uri.name;
		
		setBounds(this.center);
		setBorder();
	}
	
	public FileContentShape(FileURI uri, Point2df[] center){
		this(uri, Color.BLACK, center);
	}
	
	public FileContentShape changeColor(FileContentShape s, Color c){
		FileContentShape fcs = clone();
		fcs.color = c;
		return fcs;
	}
	
	private void setBorder(){
		float x = center.x-(WIDTH/2);
		float y = center.y-(HEIGHT/2);
		
		length = (WIDTH/POINT_INTERVAL)*4 + 1;
		points = new Point2df[length];
		points[0] = new Point2df(x, y);
		
		for(int i=0; i<4; i++){
			for(int j=0; j<WIDTH/POINT_INTERVAL && 0==i%2 ; j++){
				if(i<2)
					x += POINT_INTERVAL;
				else
					x -= POINT_INTERVAL;
				points[WIDTH/POINT_INTERVAL*i+j+1] = new Point2df(x, y);
			}
			
			for(int j=0; j<HEIGHT/POINT_INTERVAL && 1==i%2; j++){
				if(i<2)
					y += POINT_INTERVAL;
				else
					y -= POINT_INTERVAL;
				points[WIDTH/POINT_INTERVAL*i+j+1] = new Point2df(x, y);
			}
		}
	}
	
	private void setBounds(Point2df center){
		lt.x = center.x - WIDTH/2.0f;
		lt.y = center.y + HEIGHT/2.0f;
		rb.x = center.x + WIDTH/2.0f;
		rb.y = center.y - HEIGHT/2.0f;
	}	
	
	@Override
	public void paint(GuiAdapter gui){
		gui.drawImage(FileAdapter.getDefaultFileImagePath(), new Point2df(center.x, center.y+5), IMAGE_SIZE, IMAGE_SIZE);
		gui.drawString(text, ALIGN.CENTER_BOTTOM, new Point2df(lt.x + WIDTH/2, rb.y), color);
	}
	
	@Override
	public String toString(){
		return "FileContentShape-"+id+"("+uri.toString();
	}
	
	@Override
	public FileContentShape clone(){
		return (FileContentShape)super.clone();
	}	
}
