/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.util.ArrayList;

import cf.api.ApplicationKnowledge;
import cf.api.CyberApp;
import cf.api.CyberNode;


/*
	  EditApp handles dk based on the ordering defined below to limit 
	the volume of dk.
	  In the case of incomparable dks (i.e., dt_dk \not ⊂ dt_dk and
	dt_dk \not ⊃ dt_dk ), EditApp posts a new DeleteKnowledge
	dk(..., dt_dk, ...) whose dt_dk is dt_dk1 ∪ ... ∪ dt_dkn to replace
	dk1, ..., dkn, where n is the number of local dk, using O3.
*/
public class EditApp extends CyberApp{
	ArrayList<DeleteKnowledge> dks = null;
	
	private AppKnowledgeBase appKB;
	
	public EditApp(CyberNode node){
		super(node);
		dks = new ArrayList<DeleteKnowledge>();
		
		appKB = new AppKnowledgeBase(){
			public void handleKnowledge(ApplicationKnowledge k){
				if(k instanceof DeleteKnowledge)
					handleDeleteKnowledge((DeleteKnowledge)k);
			}
		};
	}
	
	@Override
	public void handleKnowledge(ApplicationKnowledge k){
		if(k instanceof DeleteKnowledge)
			appKB.updateKnowledge(k);
	}
	
	synchronized private void handleDeleteKnowledge(DeleteKnowledge k){
		//System.out.println("Received-EditApp "+k);
		dks.add(k);
		
		if(dks.size() > 1){	// if there is more than 1 DeleteKnowledge, post merge DeleteKnowledge
			DeleteKnowledge newdk = dks.get(0);
			for(int i=1; i<dks.size(); i++)
				newdk = merge(dks.get(i), newdk);
			dks = new ArrayList<DeleteKnowledge>();
			postKnowledge(newdk);
		}
	}
	
	public DeleteKnowledge merge(DeleteKnowledge d1, DeleteKnowledge d2){
		DeletingTable t = d1.getDeletingTable().merge(d2.getDeletingTable());
		return new DeleteKnowledge(t, this);
	}
}
