/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import cf.api.*;


/*
	  EditableKnowledge — ShapeKnowledge, GraphKnowledge, and ContentKnowledge —
	is created, edited, and deleted when users concurrently edit the diagram on
	their screen. Each EditableKnowledge ek contains a creation time ct and an
	identifier uuid, used to define the following partial order rule.
	 
	O1 : ek(..., ct_ek, ..., uuid_ek ) \prec ek(..., ct_ek , ..., uuid_ek )
	     if ct_ek ≤ ct_ek .
	 
	  Knowledge dissemination in the cyber-framework will then replace and
	discard all instances of ek if their creation time is earlier than that
	of a new unit of knowledge. This is a very typical aspect of a replacement
	ordering.  When a user deletes a node or edge in a map, DeleteKnowledge dk
	is generated to discard EditableKnowlege ek that is associated with the
	corresponding node or edge. 
*/
public class EditableKnowledge extends ApplicationKnowledge implements Cloneable{
	private static final long serialVersionUID = 1L;
	public String id;
	
	public EditableKnowledge(String id, CyberApplication app){
		super(app);
		this.id = id;
	}
	
	public EditableKnowledge(String id, double creationTime, CyberApplication app){
		super(app.name(), creationTime, Double.POSITIVE_INFINITY);
		this.id = id;	
	}
	
	protected EditableKnowledge(String id, String creator, double creationTime, double expirationTime){
		super(creator, creationTime, expirationTime);
		this.id = id;
	}
	
	@Override
	public boolean related(ApplicationKnowledge k){
		if(k.getClass() == getClass()){
			if(((EditableKnowledge)k).id.equals(id))
				return true;
		}else if(k instanceof DeleteKnowledge)
			return true;
		return false;
    }

	@Override
    public boolean subsumes(ApplicationKnowledge k){
        return false;
    }

	@Override
    public boolean replaces(ApplicationKnowledge k){
    	// Currently, sumsumes() doesn't remove the knowledge.
    	// So check subsuming condition here.
    	if(k.getClass() == getClass() && k.creationTime() == creationTime())
        	return true;
    		
        if(k.getClass() == getClass() && creationTime() > k.creationTime())
        	return true;
        
        return false;
    }
    
	@Override
    public void onReplaced(){
    	//System.out.println("EditableKnowledge "+toString()+" is replaced.");
    }
	
	@Override
	public EditableKnowledge clone(){
		try {
			return (EditableKnowledge)super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			return null;
		}
	}
}
