/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.URL;

import cf.common.Param;

public class FileAdapter {
	public enum FILE_TYPE {CONTENT, LOG, RESOURCE};
	
	// Home directory 
	private String HOME = "";
	
	// Resource diectory
	private String RES = "res";
	
	// Directory for Files received
	private String FILES = "files";
	
	// File image path
	private static String fileImageAbsPath;
	private static String DEFAULT_FILE_IMAGE = "file.png";	//Currently, you have to put this file manually.
	
	private static final int BUF_SIZ = 1024;
	
	public FileAdapter() throws Exception{
		String logHome = Param.LoggingDirectory+File.separator+"brainmap";
		  
		if(logHome.equals("") || !(new File(logHome).isDirectory()))
			logHome = System.getProperty("java.io.tmpdir")+File.separator+"brainmap";
		
		// Make directories
		mkdirs(logHome);
		fileImageAbsPath = RES+File.separator+DEFAULT_FILE_IMAGE;
		/*
		if(null == fileimagePath)
			fileImageAbsPath = RES+File.separator+DEFAULT_FILE_IMAGE;
		else
			fileImageAbsPath = fileimagePath;*/
	}
	
	private void mkdirs(String home) throws Exception{
		HOME = home;
		RES = HOME+File.separator+RES;
		FILES = HOME+File.separator+FILES;
		
		File h = new File(HOME);
		if(h.isFile())
			throw new Exception("Failed to create Home directory, "+ home + ". File "+home+" already exists.");
		else{
			File res = new File(RES);
			File files = new File(FILES);
			
			h.mkdirs();
			res.mkdirs();
			files.mkdirs();
		}
	}
	
	public String getRelativePath(FILE_TYPE type, String abspath){
		File f = new File(abspath);
		String name = f.getName();
		
		switch(type){
			case CONTENT:
				return getFilesDir()+File.separator+name;
			case RESOURCE:
				return getResourceDir()+File.separator+name;
			default:
				return null;
		}
	}
	
	public String getHome(){
		return HOME;
	}
	
	public String getResourceDir(){
		return RES;
	}
	
	public String getFilesDir(){
		return FILES;
	}
	
	public static String getDefaultFileImagePath(){
		return fileImageAbsPath;
	}
	
	public boolean isUnderFilesDir(String absPath){
		if(absPath.startsWith(getFilesDir()))
			return true;
		return false;
	}

	// If it failed to store file, return false.
	// But if the file is already stored or it succeed to store file, return true.
	public boolean store(FileURI uri, String relpath, byte[] data) {
		String abspath = getFilesDir()+File.separator+relpath;
		if(isContentStored(uri, relpath))
			return true;
		
		// Avoid duplicated file name
		String tmppath = getNewFileName(abspath);
		
		// Store
		File f = new File(tmppath);
		try {
			f.createNewFile();
			FileOutputStream fos = new FileOutputStream(f);
			
			fos.write(data, 0, data.length);
			
			fos.flush();
			fos.close();
		} 
		catch (Exception e) {
			System.err.println("FATAL : Failed to store file.. "+ tmppath);
			e.printStackTrace();
			return false;
		}
		
		if(f.isFile()){
			// Restore file name TODO : what's this
			if(!abspath.equals(tmppath))
				changeFileName(tmppath, abspath);
			return true;
		}
		
		return false;
	}
	
	public void copyFile(String srcAbsPath, String dstAbsPath){
		File src = new File(srcAbsPath);
		/*File dst = new File(dstAbsPath);

		try {
			int read = -1;
			byte[] buf = new byte[BUF_SIZ];
			dst.createNewFile();
			FileInputStream srcIn = new FileInputStream(src);
			FileOutputStream dstOut = new FileOutputStream(dst);
			
			while(-1 != (read = srcIn.read(buf)))
				dstOut.write(buf, 0, read);
			dstOut.flush();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		try {
			copyFile(new FileInputStream(src), dstAbsPath);
		} catch (FileNotFoundException e) {e.printStackTrace();	}
	}
	
	public void copyFile(InputStream srcIn, String dstAbsPath){
		File dst = new File(dstAbsPath);
		try{
			int read = -1;
			byte[] buf = new byte[BUF_SIZ];
			dst.createNewFile();
			FileOutputStream dstOut = new FileOutputStream(dst);
			
			while(-1 != (read = srcIn.read(buf, 0, BUF_SIZ)))
				dstOut.write(buf, 0, read);
			dstOut.flush();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	// Generate new file name which is not duplicated with others.
	// If it failed to generated non duplicated file name, return "".
	// Otherwise, return none duplicated file name
	public String getNewFileName(String abspath){
		String tmppath = abspath;
		if(new File(abspath).isFile()){
			int i=0;
			while((abspath.equals(tmppath) || new File(tmppath).isFile()) && i < Integer.MAX_VALUE){
				tmppath = abspath+i;
				i++;
			}
		}
		if(new File(tmppath).isFile())
			return "";
		return tmppath;
	}
	
	// Search a file from HOME. If there is no file whose name is filename, return null. 
	// Otherwise it returns absolute path of the file
	// files -> res -> log
	public String search(String filename){
		String ret = search(filename, new File(getFilesDir()));
		if(null != ret)
			return ret;
		ret = search(filename, new File(getResourceDir()));
		if(null != ret)
			return ret;
		return null;
	}
	
	// If it find the file, returns the absolute path of the file. Otherwise return null
	private String search(String key, File dir){
		File[] files = dir.listFiles();
		for(int index=0; index<files.length; index++){
			if(files[index].isFile() && files[index].getName().equals(key))
				return files[index].getAbsolutePath();
			else if(files[index].isDirectory()){
				String ret = search(key, dir);
				if(null != ret)
					return ret;
			} 
		}
		return null;
	}
	
	private boolean isContentStored(FileURI uri){
		String ret = search(uri.name, new File(getFilesDir()));
		if(null == ret)
			return false;
		return true;
	}
	
	private boolean isContentStored(FileURI uri, String relpath){
		String abspath = getFilesDir()+File.separator+relpath;
		File f = new File(abspath);
		if(f.isFile()){
			if(f.getName().equals(uri.name) && f.length()==uri.size && f.lastModified()==uri.lastFileModifiedTime)
				return true;
		}
		return false;
	}
	
	private void changeFileName(String srcAbsPath, String dstAbsPath){
		File src = new File(srcAbsPath);
		File dst = new File(dstAbsPath);
		if(dst.isFile())
			dst.delete();
		src.renameTo(dst);
	}	
}
