/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;

import cf.api.*;
import cf.app.brainmap.events.*;


/*
     GraphApp edits and extends the graph data of a map by handling and disseminating 
   GraphKnowledge gk. It deletes gk by posting dk when a GraphDeleteEvent is 
   delivered
*/
public class GraphApp extends CyberApp{
	public Hashtable<String,HashSet<String>> graph;
	public Hashtable<String,GraphKnowledge> graphKnowledges;
	
	private AppKnowledgeBase appKB;
	
	public GraphApp(CyberNode node){
		super(node);
		graph = new Hashtable<String, HashSet<String>>();
		graphKnowledges = new Hashtable<String,GraphKnowledge>();
		
		appKB = new AppKnowledgeBase(){
			public void handleKnowledge(ApplicationKnowledge k){
				if(k instanceof GraphKnowledge)
					postEvent(new KnowledgeAcceptanceEvent(k , BrainmapRealWorld.brainmapApp));
			}
		};
	}
	
	@Override
	public void handleEvent(ApplicationEvent e){
		if(e instanceof KnowledgeDisseminationEvent)
			postKnowledge((GraphKnowledge)((KnowledgeDisseminationEvent) e).k);
		
		else if(e instanceof KnowledgeAcceptanceEvent){
			GraphKnowledge k = (GraphKnowledge)((KnowledgeAcceptanceEvent) e).k;
			if(GraphKnowledge.TYPE.NODE == k.type)
				addNode(k.id);
			else
				addEdge(k.n1, k.n2);
			graphKnowledges.put(k.id, k);
		}
		/*if(e instanceof DisseminationEvent){
			GraphKnowledge k = (GraphKnowledge)((DisseminationEvent) e).k;
			
			if(DisseminationEvent.SOURCE.GLOBAL == ((DisseminationEvent)e).source){
				if(GraphKnowledge.TYPE.NODE == k.type)
					addNode(k.id);
				else
					addEdge(k.n1, k.n2);
				graphKnowledges.put(k.id, k);
			}else
				postKnowledge(k);
			
		}*/else if(e instanceof GraphDeleteEvent){
			//System.out.println("Received-graphApp " + e);
			deleteGraph(((GraphDeleteEvent)e).id);
		}
	}
	
	@Override
	public void handleKnowledge(ApplicationKnowledge k){
		if(k instanceof GraphKnowledge)
			appKB.updateKnowledge(k);
	}
	
	private void deleteGraph(String id){		
		GraphKnowledge k = graphKnowledges.get(id);
		
		if(null == k)
			return;
		
		postKnowledge(new DeleteKnowledge(k, this));
	}
	
	private void printGraph(){
		System.out.println("Graph");
		Enumeration<String> e = graph.keys();
		
		while(e.hasMoreElements()){
			String key = e.nextElement();
			System.out.println("  Node:"+key);
			HashSet<String> edges = graph.get(key);
			for(String n2 : edges)
				System.out.println("    "+key+" -> "+n2);
		}
	}
	
	private void addNode(String node){		
		if(null == graph.get(node))
			graph.put(node, new HashSet<String>());
		/*else
			System.err.println("GraphApp::addNode:: Failed to add node "+node);*/
	}
	
	private boolean addEdge(String n1, String n2){
		
		if(null == n1 || n2 == null)
			return true;
		
		if(null == graph.get(n1))
			addNode(n1);
		if(null == graph.get(n2))
			addNode(n2);
		
		HashSet<String> edgeset1 = graph.get(n1);
		HashSet<String> edgeset2 = graph.get(n2);
		
		if(edgeset1.contains(n2) || edgeset2.contains(n1))
			return false;
		
		edgeset1.add(n2);
		edgeset2.add(n1);
		
		return true;
	}
	
	public boolean containsEdge(String n1, String n2){
		//this.printGraph();
		
		if(null != n1 && null != graph.get(n1)){
			HashSet<String> edgeset = graph.get(n1);
			/*for(String s : edgeset){
				System.out.println("  "+n1+" -> "+s);
			}
			System.out.println("  "+n1+" contains "+n2+"? "+edgeset.contains(n2));*/
			if(edgeset.contains(n2)){
				//System.out.println("CONTAINS! "+n1+" , "+n2);
				return true;
			}
				
		}else if(null != n2 && null != graph.get(n2)){
			HashSet<String> edgeset = graph.get(n2);
			/*for(String s : edgeset){
				System.out.println("  "+n2+" -> "+s);
			}
			System.out.println("  "+n2+" contains "+n1+"? "+edgeset.contains(n1));*/
			if(edgeset.contains(n1)){
				//System.out.println("CONTAINS! "+n1+" , "+n2);
				return true;
			}
		}
		
		//System.out.println("NOT CONTAINS! "+n1+" , "+n2);
		return false;
	}
	
	public void removeEdge(String n1, String n2){
		//System.out.println("removeEdge::"+n1+" , "+n2);
		
		if(n1 != null && null != graph.get(n1)){
			HashSet<String> edgeset1 = graph.get(n1);
			edgeset1.remove(n2);
		}
		
		if(n2 != null && null != graph.get(n2)){
			HashSet<String> edgeset2 = graph.get(n2);
			edgeset2.remove(n1);
		}
	}
	
	public void removeNode(String n){
		//System.out.println("removeNode::"+n);
		
		if(null != n && null != graph.get(n)){
			HashSet<String> edges = graph.get(n);
			
			for(String n2 : edges){
				HashSet<String> edges2 = graph.get(n2);
				if(null != edges2)
					edges2.remove(n);
			}
			
			graph.remove(n);
		}
	}
}
