/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

public class GraphKnowledge extends EditableKnowledge implements Waitable{
	public enum TYPE {NODE, EDGE}
	
	public String n1, n2;	// node's id or value
	public String value = null;
	public TYPE type;
	
	private static final long serialVersionUID = 1L;	
	
	public GraphKnowledge(String id, TYPE type, String n1, String n2){
		super(id, BrainmapRealWorld.graphApp);
		this.type = type;
		this.n1 = n1;
		this.n2 = n2;
	}
	
	public GraphKnowledge(String id, TYPE type, String n1, String n2, double creationTime){
		super(id, creationTime, BrainmapRealWorld.graphApp);
		this.type = type;
		this.n1 = n1;
		this.n2 = n2;		
	}
	
	@Override
	public String toString(){
		if(type == TYPE.EDGE)
			return ("GraphKnowledge(EDGE) "+id+" : "+n1+" -> "+n2 );
		else
			return ("GraphKnowledge(NODE) "+id );
	}
	
	@Override
	public boolean equals(Object o){
		if(o instanceof GraphKnowledge){
			if(((GraphKnowledge)o).id == id)
				return true;
		}
		return false;			
	}	
	
	@Override
	public void onReplaced(){
		if(null == BrainmapRealWorld.graphApp)
			return;
		
		if(TYPE.NODE == type)
			BrainmapRealWorld.graphApp.removeNode(id);
		else
			BrainmapRealWorld.graphApp.removeEdge(n1, n2);
	}

	@Override
	public String id() {
		return id;
	}
}
