/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.util.ArrayList;
import java.util.Hashtable;


public interface GuiAdapter {	
	// Drawing on drawing pane
	public void drawString(String str, ALIGN align, Point2df p, Color color);
	public void drawLine(Point2df[] points, int length, Color color);
	public void drawImage(String path, Point2df center, float width, float height);
	public void drawFilledPolygon(Point2df[] points, int length, Color lineColor, Color fillColor);
	public void drawFilledRect(Point2df lt, float width, float height, Color color);
	public void drawFilledCircle(Point2df center, float radius, Color color);
	
	// Set visibility of drawing pane
	public void hideDrawingPane();
	public void showDrawingPane(String title);
		
	// ShapeKnowledge
	public int shapeNum();
	public void removeAllShapeKnowledge();
	public boolean addShapeKnowledge(ShapeKnowledge s);
	public ShapeKnowledge getShapeKnowledge(String id);
	public void removeShapeKnowledge(ShapeKnowledge s);
	public ShapeKnowledge selectShapeKnowledge(String id);
	
	// Temp Shape
	public void removeTempShape();
	public void addTempShape(Shape s);	
	
	// Projection
	public void startToMoveScreen();
	public void finishToMoveScreen();
	public void moveScreen(Point2df vector);
	public Point2df projectToMap(Point2df p);
	public Point2df projectToScreen(Point2df p);
	
	// Handle Other GUI Components
	public void attachFile();
	public void enableKeyInput();
	public void disableKeyInput();
	public void popupMsg(String msg);
	public void popupmenu(Point2df p);
	public void sessionbrowser();
	public void updateSessions(ArrayList<Session> sessions);
	
	// File control result callback
	public void successToAttachFile();
	public void successToSaveSession();
	public void failedToSaveSession();
	public void successToLoadSession();
	public void failedToLoadSession(String errmsg);
}
