/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.io.Serializable;
import java.util.Iterator;

import cf.real.CyberEngineIdList;
import cf.util.Dist;

public class Session implements Cloneable, Serializable{
	private static final long serialVersionUID = 1L;
	
	public String id;
	public String title;
	public CyberEngineIdList engines;
	
	public Session(String title, CyberEngineIdList engines){
		Dist dist = new Dist(System.currentTimeMillis());
		id = Long.toString(dist.rand().nextLong());
		this.title = title;
		this.engines = engines;
	}
	
	public Session(String title, long creator){
		this(title, new CyberEngineIdList());
		addEngine(creator);
	}
	
	public boolean addEngine(long engine){
		return engines.add(engine);
	}
	
	public boolean removeEngine(long engine){
		return engines.remove(engine);
	}
	
	public void removeAllEngines(){
		engines = new CyberEngineIdList();
	}
	
	@Override
	public boolean equals(Object o){
		if(o instanceof Session){
			Session s = (Session)o;
			if(id.equals(s.id))// && s.contains(engines) && contains(s.engines))
				return true;
		}
		return false;
	}
	
	public boolean contains(long engine){
		return engines.contains(engine);
	}
	
	public boolean contains(CyberEngineIdList list){
		for(long e : list){
			if(!contains(e))
				return false;
		}
		
		return true;
	}
	
	public int size(){
		return engines.size();
	}
	
	public Iterator<Long> engines(){
		return this.engines.iterator();
	}
	
	@Override
	public String toString(){
		return "Session["+id+"]("+engines.size()+") "+title;
	}
	
	@Override
	public Session clone(){
		try {
			return (Session)super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			return null;
		}
	}
}

class SessionKnowledge extends EditableKnowledge implements Cloneable{
	private static final long serialVersionUID = 1L;
	
	public Session session;
	
	public SessionKnowledge(Session session){
		super(session.id, BrainmapRealWorld.brainmapApp);
		
		this.session = session;
	}
	
	@Override
	public void onReplaced(){
		System.out.println(BrainmapRealWorld.brainmapApp+","+id);
		BrainmapRealWorld.brainmapApp.removeSession(id);
	}
	
	@Override
	public String toString(){
		return "SessionKnowledge "+session.toString();
	}
}

class SessionBeaconKnowledge extends EditableKnowledge{
	private static final long serialVersionUID = 1L;
	
	public SessionBeaconKnowledge(String id){
		super(id, BrainmapRealWorld.brainmapApp);
	}
	
	@Override
	public String toString(){
		return "SessionBeacon - "+id;
	}
}
