/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.io.Serializable;

import cf.util.Dist;

public interface URI{
	public boolean equals(Object o); 
	public String toString();
	public String id();
}

class FileURI implements URI, Serializable, Cloneable{
	private static final long serialVersionUID = 1L;
	
	private final String id = Long.toString(Dist.dist.rand().nextLong());
	public String name;
	public long size;
	
	/* The last modified time of the file which is matched with this FileURI
	 * A long value representing the time the file was last modified, 
	 * measured in milliseconds since the epoch (00:00:00 GMT, January 1, 1970), 
	 * or 0L if the file does not exist or if an I/O error occurs*/
	public long lastFileModifiedTime;
	
	public FileURI(String name, long size, long lastFileModifiedTime){
		this.name = name;
		this.size = size;
		this.lastFileModifiedTime = lastFileModifiedTime;
		//System.out.println("File size : "+size+"   Time : "+lastFileModifiedTime);
	}
	
	public boolean equals(Object o){
		if(o instanceof FileURI){
			FileURI fileuri = ((FileURI) o);
			if(id().equals(fileuri.id()) && name.equals(fileuri.name)
					&& size == fileuri.size && lastFileModifiedTime==fileuri.lastFileModifiedTime)
				return true;
		}
		return false;
	}
	
	public boolean related(FileURI fileuri){
		if(id().equals(fileuri.id()) && name.equals(fileuri.name))
			return true;
		return false;
	}
	
	// Update file URI for modified File Content
	public void modify(long size, long lastModifiedTime){
		this.size = size;
		this.lastFileModifiedTime = lastModifiedTime;
	}
	
	@Override
	public String id(){
		return id;
	}
	
	@Override
	public String toString(){
		return "FileURI("+name+","+size+","+lastFileModifiedTime+")";
	}
	
	@Override
	public FileURI clone(){
		try {
			return (FileURI)super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			return null;
		}
	}
}
