/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap;

import java.util.HashSet;
import java.util.Hashtable;


/* 
	  WaitingQueue makes sure that a pair of ApplicationKnowledge is delivered.
	Inserted ApplicationKnowledge would be in the Queue until an ApplicationKnowledge
	having same id is inserted in "waintingQueue". In instance, if you want to make a pair 
	with type A and B, you can create two WaitingQueues respectively and then call setWaitingQueue()
	method like A.setWatingQueue(B), B.setWatingQueue(A).  
 	  Whenever ApplicationKnowledge is inserted, WaitingQueue searches an ApplicationKnowledge
 	having same id in "watingQueue". If there is an ApplicationKnowledge having same id(), 
 	WatingQueue calls onMet() method to notify.
 	  If checkHistory is true, it checks history keeping all ids of inserted ApplicationKnowledge.If 
 	there is an entry having same id, WaitingQueue calls onHistory() method to notify.
*/
public class WaitingQueue extends Hashtable<String, Waitable>{
	private static int DEFAULT_CAPACITY = 15;
	WaitingQueueListener listener = null;
	WaitingQueue watingQueue= null;
	boolean checkHistory;
	String name;
	
	public HashSet<String> history;
	
	public WaitingQueue(String name, boolean checkHistory){
		super(DEFAULT_CAPACITY);
		this.name = name;
		history = new HashSet<String>();
		this.checkHistory = checkHistory;
	}

	public void setWaitingQueue(WaitingQueue q){
		//watingQueues = new WaitingQueue[1];
		watingQueue = q;
	}
	/*
	public void setWaitingQueues(WaitingQueue[] waitingFor){
		this.waitingFor = waitingFor;
	}*/
	
	public void setWaitingQueueListener(WaitingQueueListener listener){
		this.listener = listener;
	}
	
	public void add(Waitable w){
		if(null != watingQueue){
			// Check queues first
			synchronized(watingQueue){
				//for(int index=0; index<waitingFor.length; index++){
					Waitable target = watingQueue.get(w.id());
					if(null != target){
						watingQueue.history.add(w.id());
						history.add(w.id());
						watingQueue.remove(w.id());
						remove(w.id());
						onMet(w, target);
						return;
					}
				//}
			}
			
			// Check history
			if(checkHistory && history.contains(w.id())){
				onHistory(w);
				return;
			}
			
			this.put(w.id(), w);
		}
	}
	
	public String toString(){
		String result = "[";
		for(String key: this.keySet())
			result = result+","+key;
		return result+"]";
	}
	
	private void onMet(Waitable w1, Waitable w2){
		if(null == listener)
			return;
		listener.onMet(w1, w2);
	}
	
	private void onHistory(Waitable w){
		if(null == listener)
			return;
		listener.onHistory(w);
	}
}

interface Waitable {
	public String id();
}

interface WaitingQueueListener {
	public void onMet(Waitable w1, Waitable w2);
	public void onHistory(Waitable w);
}
