/*

   Copyright 2010, 2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package com.sri.brainmap;

import cf.app.brainmap.*;
import cf.app.brainmap.events.*;
import cf.app.brainmap.events.KeyEvent;
import cf.app.brainmap.Color;

import java.io.File;
import java.util.*;

import com.sri.brainmap.FileBrowserActivity.RESULT;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
//import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.*;
import android.view.inputmethod.InputMethodManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Toast;


public class AndroidGuiAdapter extends View implements GuiAdapter, View.OnTouchListener/*, View.OnKeyListener*/ {
	
	// Font size of text to draw
	private static float FONT_SIZE = 15.0f;
	
	// Line with on drawing pane
	private static final float STROKE_WITH = 2.0f;
	
	// GUI adapter state for activity control
	private enum STATE {DRAWING, BROWSING_SESSION, LOAD_SESSION, 
		SAVE_SESSION, ATTACH_FILE, ASK_TERMINATION, POSTING_SESSION_BROWSER};
	private STATE state = null;
	
	// For Keyboard and Key control
	private InputMethodManager im;
	private int keyinputCounter = 0;
	private static final int FUNCTION_KEYS = 0;
	
	// For updating GUI
	private Handler messageHandler;
	
	// Shape and ShapeKnowledge
	private Shape tempShape = null;
	private Hashtable<String, ShapeKnowledge> shapes;
	
	private Activity activity;
	private Canvas c = null;
	
	// For projection and Screen moving
	private Point2df maplt;
	private Point2df startlt = null;
	
	// For Shape selection
	private Color selectedShapeColor = null;
	private ShapeKnowledge selectedShapeKnowledge = null;
	
	// Other components
	private PopupMenu popupMenu = null;
	private SessionBrowserView sessionBrowser = null;
	
	// For bitmap caching
	Hashtable<String, Bitmap> bitmapTable;	
	
	public AndroidGuiAdapter(Activity activity){
		super(activity);
		this.activity = activity;
		
		shapes = new Hashtable<String, ShapeKnowledge>();
		bitmapTable = new Hashtable<String,Bitmap>();
		
		messageHandler = generateMsgHandler();
		maplt = new Point2df(-getWidth()/2, getHeight()/2);
		im = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
		
		setOnTouchListener(this);
		sessionBrowser = new SessionBrowserView(activity, this);
		
		activity.setContentView(sessionBrowser);
	}

	private void browsingSession(){
		state = STATE.BROWSING_SESSION;
	}
	
	private void createSession(Session session){
		state = STATE.DRAWING;
		this.removeAllShapeKnowledge();
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.CREATE, session, false, 0.0, BrainmapRealWorld.brainmapApp));
	}
	
	private void joinSession(Session session){
		state = STATE.DRAWING;
		this.removeAllShapeKnowledge();
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.JOIN, session, false, 0.0, BrainmapRealWorld.brainmapApp));
	}
	
	private void exitSession(){
		state = STATE.POSTING_SESSION_BROWSER;
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.EXIT, BrainmapRealWorld.brainmapApp));
		app.postEvent(new SessionEvent(SessionEvent.TYPE.SHOW, app));
	}
	
	private void saveSession(){
		state = STATE.SAVE_SESSION;
		filebrowser("Save");
	}
	
	private void cancelToSaveSession(){
		state = STATE.DRAWING;
		popupMsg("Canceled");
	}
	
	@Override
	public void successToSaveSession(){
		state = STATE.DRAWING;
		popupMsg("Session Saved");
	}
	
	@Override
	public void failedToSaveSession(){
		state = STATE.DRAWING;
		popupMsg("Failed to Save Session");
	}
	
	private void loadSession(){
		state = STATE.LOAD_SESSION;
		filebrowser("Load");
	}
	
	private void cancelToLoadSession(){
		state = STATE.POSTING_SESSION_BROWSER;
		popupMsg("Canceled");
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.SHOW, app));
	}
	
	@Override
	public void failedToLoadSession(String errmsg){
		state = STATE.POSTING_SESSION_BROWSER;
		popupMsg(errmsg);
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.SHOW, app));
	}
	
	@Override
	public void successToLoadSession(){
		state = STATE.DRAWING;
	}
	
	@Override
	public void attachFile(){
		state = STATE.ATTACH_FILE;
		filebrowser("Attach");
	}
	
	private void cancelToAttachFile(){
		state = STATE.DRAWING;
		popupMsg("Canceled");
	}
	
	@Override
	public void successToAttachFile(){
		state = STATE.DRAWING;
	}
	
	private void terminate(){
		activity.finish();
	}
	
	private void askTermination(){
		state = STATE.ASK_TERMINATION;
		AlertDialog.Builder ab = new AlertDialog.Builder(activity);
		ab.setMessage("Do you want to Terminate?");
		ab.setPositiveButton("Yes", new DialogInterface.OnClickListener(){
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				terminate();
			}
		}).setNegativeButton("No", new DialogInterface.OnClickListener(){
			@Override
			public void onClick(DialogInterface dialog, int which) {
				state = STATE.BROWSING_SESSION;
			}
		});
		
		ab.show();
	}
	
	@Override
	public void sessionbrowser(){
		state = STATE.POSTING_SESSION_BROWSER;
		Message msg = Message.obtain(messageHandler);
		Bundle data = new Bundle();
		data.putCharSequence("Type", "Visible");
		data.putCharSequence("Data", "SessionBrowser");
		data.putCharSequence("Title", "Brainstorming Sessions");
		msg.setData(data);
		msg.sendToTarget();
		browsingSession();		
	}
	
	@Override
	public void removeAllShapeKnowledge(){
		shapes = new Hashtable<String, ShapeKnowledge>();
	}
	
	@Override
	public void updateSessions(ArrayList<Session> sessions){
		if(STATE.BROWSING_SESSION != state)
			return;
		
		sessionBrowser.refreshList(sessions);
	}
	
	@Override
	public void showDrawingPane(String title){
		
		Message msg = Message.obtain(messageHandler);
		Bundle data = new Bundle();
		data.putCharSequence("Type", "Visible");
		data.putCharSequence("Data", "DrawingPane");
		data.putCharSequence("Title", title);
		msg.setData(data);
		msg.sendToTarget();
	}
	
	@Override
	public void hideDrawingPane(){
		Message msg = Message.obtain(messageHandler);
		Bundle data = new Bundle();
		data.putCharSequence("Type", "Invisible");
		data.putCharSequence("Data", "DrawingPane");
		msg.setData(data);
		msg.sendToTarget();
	}
	
	private void filebrowser(String selectButtonName){
		// Start file browser activity
		Intent intent = new Intent(activity, FileBrowserActivity.class);
		activity.startActivityForResult(intent, BrainmapActivity.FILEBROWSER_ACTIVITY_REQUEST_CODE);
	}
	
	public void onFilebrowserActivityResult(Intent intent){
		FileBrowserActivity.RESULT result = (FileBrowserActivity.RESULT)intent.getExtras().get("Result");
		
		switch(result){
		case FILE_SELECTED:
			String path = intent.getExtras().getString("Path");
			
			SketchApp app = BrainmapRealWorld.sketchApp;
			
			switch(state){
			case DRAWING:
			case BROWSING_SESSION:
				break;
			
			case SAVE_SESSION:
				app.postEvent(new MapFileEvent(MapFileEvent.TYPE.SAVE, path));
				break;
				
			case LOAD_SESSION:
				app.postEvent(new MapFileEvent(MapFileEvent.TYPE.LOAD, path));
				// In the sketchapp, failed or success to seccion is called
				break;
				 
			case ATTACH_FILE:
				app.postEvent(new FilePathEvent(path));
				break;
			}
			break;
			
		case CANCEL:
			switch(state){
			case DRAWING:
			case BROWSING_SESSION:
				break;
			
			case SAVE_SESSION:
				cancelToSaveSession();
				break;
				
			case LOAD_SESSION:
				cancelToLoadSession();
				break;
				
			case ATTACH_FILE:
				cancelToAttachFile();
				break;
			}
			break;
		}
	}
	
	public void onSessionbrowserResult(SessionBrowserView.RESULT result, Session session){
		SketchApp app = BrainmapRealWorld.sketchApp;
		
		switch(result){
		case CREATE_SESSION:
			createSession(session);
			break;
			
		case JOIN_SESSION:
			joinSession(session);
			break;
			
		case LOAD_SESSION:
			loadSession();
			break;
		}
	}
	
	public void showExitSessionDialog(){
		AlertDialog.Builder ab = new AlertDialog.Builder(activity);
		ab.setMessage("Do you want to Exit the Discussion?");
		ab.setPositiveButton("Yes", new DialogInterface.OnClickListener(){
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				exitSession();
			}
		}).setNegativeButton("No", new DialogInterface.OnClickListener(){
			@Override
			public void onClick(DialogInterface dialog, int which) {
			}
		});
		
		ab.show();
	}	
	
	public void showSaveDialog(){
		AlertDialog.Builder ab = new AlertDialog.Builder(activity);
		ab.setMessage("Do you want to save map?");
		ab.setPositiveButton("OK", new DialogInterface.OnClickListener(){
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				saveSession();
				
			}
		}).setNegativeButton("No", new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				showExitSessionDialog();
			}
		});
		ab.show();
	}
	
	public boolean onKey(View v, int keyCode, android.view.KeyEvent event) {
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		
		keyinputCounter = (keyinputCounter+1)%2;
		if(keyinputCounter == 1){
			int key = event.getUnicodeChar();
			//System.out.println("Key!!!::"+key+"    keycode="+keyCode);
			
			switch(key){
				case FUNCTION_KEYS:
					switch(keyCode){
					case 4:	//back
						switch(state){
						case DRAWING:
							if(null != popupMenu)
								closePopupMenu();
							else
								showSaveDialog();	//save or exit session
							break;
							
						case BROWSING_SESSION:
							askTermination();
							break;
							
						case ASK_TERMINATION:
							state = STATE.BROWSING_SESSION;
							break;
						}
						
						break;
						
					case 82://menu
					case 24://volume up
					case 25://volume down
					case 26://power button
					case 84://search
						break;
						
					default:	//BS
						int bs =SketchApp.BS;
						key = bs;
						app.postEvent(new KeyEvent(key));
						break;
					}
					break;
				default:
					app.postEvent(new KeyEvent(key));
					break;
			}
		}
		return true;
	}
	
	/*Msg structure
	 *	Type(String)	|	Data	|	title
	 *-------------------------------------------
	 *	Invalidate		|	-		|	-
	 * 	Toast			|	String	|	-
	 *  Visible			|	String	|	String
	 *  Invisible		|	String	|	-
	*/
	private Handler generateMsgHandler(){
		return new Handler(){
			public void handleMessage(Message msg){
				
				Bundle b = msg.getData();
				String type = b.getCharSequence("Type").toString();
				
				if(type.equals("Invalidate"))
					invalidate();
				
				else if(type.equals("Toast")){
					String str = b.getCharSequence("Data").toString();
					popupToast(str);
					
				}
				
				else if(type.equals("Visible")){
					String str = b.getCharSequence("Data").toString();
					String title = b.getCharSequence("Title").toString();
					
					if(str.equals("DrawingPane"))
						setDrawingPaneVisible(true, title);
					else if(str.equals("SessionBrowser"))
						setSessionBrowserVisible(true, title);
				}
				
				else if(type.equals("Invisible")){
					String str = b.getCharSequence("Data").toString();
					
					if(str.equals("DrawingPane"))
						setDrawingPaneVisible(false, null);
					else if(str.equals("SessionBrowser"))
						setSessionBrowserVisible(false, null);
				}
			}
		};
	}
	
	private void setDrawingPaneVisible(boolean flag, String title){
		if(flag){
			activity.setContentView(this);
			setVisibility(LinearLayout.VISIBLE);
			activity.setTitle(title);
		}else
			setVisibility(LinearLayout.GONE);
	}
	
	private void setSessionBrowserVisible(boolean flag, String title){
		
		if(flag){
			activity.setContentView(sessionBrowser);
			sessionBrowser.setVisibility(LinearLayout.VISIBLE);
			activity.setTitle(title);
		}else
			sessionBrowser.setVisibility(LinearLayout.GONE);
	}
	
	@Override
	synchronized public void onDraw(Canvas canvas){
		c = canvas;
		Enumeration e = shapes.keys();
		while(e.hasMoreElements()){
			String key = (String)e.nextElement();
			
			ShapeKnowledge k = (ShapeKnowledge)shapes.get(key);
			
			if(null == k)
				continue;
			
			if(isVisibleOnScreen(k.s))
				k.s.paint(this);
		}
		
		if(null != tempShape)
			tempShape.paint(this);
		
		if(null!=selectedShapeKnowledge)
			selectedShapeKnowledge.s.paint(this);
		
		if(null != popupMenu)
			popupMenu.paint(this);
		
		c = null;
	}
	
	@Override
	public void enableKeyInput(){
		im.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0);
		if(null != popupMenu)
			closePopupMenu();
	}
	
	@Override
	public void disableKeyInput(){
		im.toggleSoftInput(InputMethodManager.HIDE_NOT_ALWAYS, 0);
	}

	@Override
	public int shapeNum(){
		return shapes.size();
	}
	
	@Override
	synchronized public boolean addShapeKnowledge(ShapeKnowledge s){
		if(null != shapes.get(s.id)){
			System.err.println("AndroidGuiAdapter::addShapeKnowledge():Duplicated ShapeKnowledge " + s);
			return false;
		}
		
		shapes.put(s.id, s);
		redraw();
		return true;
	}
	
	@Override
	synchronized public void removeShapeKnowledge(ShapeKnowledge s){
		shapes.remove(s.id);
		redraw();
	}
	
	@Override
	public ShapeKnowledge getShapeKnowledge(String id){
		return shapes.get(id);
	}
	
	@Override
	synchronized public void addTempShape(Shape s){
		tempShape = s;
		redraw();
	}
	
	@Override
	synchronized public void removeTempShape(){
		tempShape = null;
		redraw();
	}
	
	@Override
	public ShapeKnowledge selectShapeKnowledge(String id){		
		ShapeKnowledge s = shapes.get(id);
		
		if(null == s)
			return null;
		
		selectedShapeKnowledge = s.clone();
		selectedShapeColor = s.s.color;
		selectedShapeKnowledge.s.color = Color.RED;
		
		redraw();
		return s;
	}
		
	private void redraw(){
		Message msg = Message.obtain(messageHandler);
		Bundle data = new Bundle();
		data.putCharSequence("Type", "Invalidate");
		msg.setData(data);
		msg.sendToTarget();
	}
		
	@Override
	public void moveScreen(Point2df vector){
		if(null == startlt)
			return;
		maplt = new Point2df(startlt.x+vector.x, startlt.y+vector.y);
		redraw();
	}
	
	@Override
	public void startToMoveScreen(){
		startlt = new Point2df(maplt.x, maplt.y);
	}
	
	@Override
	public void finishToMoveScreen(){
		startlt = null;
	}
		
	private boolean isVisibleOnScreen(Shape s){
		Point2df projectedLT = projectToScreen(s.lt);
		Point2df projectedRB = projectToScreen(s.rb);
		
		int width = getWidth();
		int height = getHeight();
		
		if(width<projectedLT.x || height<projectedLT.y || projectedRB.x<0 || projectedRB.y<0)
			return false;
		
		return true;
	}
	
	public Point2df projectToScreen(Point2df p){
		return new Point2df(p.x - maplt.x, maplt.y - p.y);
	}	
	
	public Point2df[] projectToScreen(Point2df[] points, int len){
		Point2df[] projected = new Point2df[len];
		
		for(int i=0; i<len; i++)
			projected[i] = projectToScreen(points[i]);
		
		return projected;
	}
	
	@Override
	public Point2df projectToMap(Point2df p){
		return new Point2df(maplt.x + p.x, maplt.y - p.y);
	}
	
	@Override
	public void drawString(String str, ALIGN align, Point2df p, Color color){
		Canvas c = getCurrentCanvas();
		Rect bounds = new Rect();
		
		Paint paint = new Paint();
		paint.setTextSize(FONT_SIZE);
		paint.setColor(shapeColor2AndroidColor(color));
		paint.getTextBounds(str, 0, str.length(), bounds);
		
		p = projectToScreen(p);
		
		float dy = (float)bounds.height();
		float y = p.y+ dy;
		
		switch(align){
		case CENTER:
			paint.setTextAlign(Align.CENTER);
			break;
		case CENTER_BOTTOM:
			y -= dy;
			paint.setTextAlign(Align.CENTER);
			break;
		case LEFT_TOP:
			paint.setTextAlign(Align.LEFT);
			break;
		case RIGHT_TOP:
			paint.setTextAlign(Align.RIGHT);
			break;
		}
		
		c.drawText(str, p.x, y, paint);
	}
	
	@Override
	public void drawLine(Point2df[] points, int length, Color color){
		Canvas c = getCurrentCanvas();
		
		Paint paint = new Paint();
		paint.setStrokeWidth(STROKE_WITH);
		paint.setColor(shapeColor2AndroidColor(color));
		Point2df[] projectedPoints = projectToScreen(points, length);
		
		for(int index=1; index<length; index++){
			Point2df p1 = projectedPoints[index-1];
			Point2df p2 = projectedPoints[index];
			c.drawLine(p1.x, p1.y, p2.x, p2.y, paint);
		}
	}
	
	@Override
	public void drawFilledPolygon(Point2df[] points, int length, Color lineColor, Color fillColor){
		Canvas c = getCurrentCanvas();
		
		Paint paint = new Paint();
		paint.setStrokeWidth(STROKE_WITH);
		paint.setColor(shapeColor2AndroidColor(lineColor));
		Point2df[] projectedPoints = projectToScreen(points, length);
		
		for(int index=1; index<length; index++){
			Point2df p1 = projectedPoints[index-1];
			Point2df p2 = projectedPoints[index];
			c.drawLine(p1.x, p1.y, p2.x, p2.y, paint);
		}
		
		//TODO : fill the polygon
		paint.setColor(shapeColor2AndroidColor(fillColor));
	}
	
	@Override
	public void drawImage(String path, Point2df center, float width, float height){
		Canvas c = getCurrentCanvas();
		
		String bitmapKey = path+"-"+width+"-"+height;
		Bitmap img = bitmapTable.get(bitmapKey);
		if(null == img){
			img = Bitmap.createScaledBitmap(BitmapFactory.decodeFile(path), (int)width, (int)height, true);
			bitmapTable.put(bitmapKey, img);
		}
		
		center = projectToScreen(center);
		float left = center.x - width/2.0F;
		float top = center.y - height/2.0F;
		
		c.drawBitmap(img, left, top, null);
	}
	
	@Override
	public void drawFilledRect(Point2df lt, float width, float height, Color color){
		Canvas c = getCurrentCanvas();
		lt = projectToScreen(lt);
		
		Paint paint = new Paint();
		
		paint.setColor(shapeColor2AndroidColor(color));
		
		c.drawRect(lt.x, lt.y, lt.x+width, lt.y+height, paint);
	}
	
	@Override
	public void drawFilledCircle(Point2df center, float radius, Color color){
		Canvas c = getCurrentCanvas();
		center = projectToScreen(center);
		
		Paint paint = new Paint();
		paint.setColor(shapeColor2AndroidColor(color));
		
		c.drawCircle(center.x, center.y, radius, paint);
	}
	
	@Override
	public void popupMsg(String msg){
		Message message = Message.obtain(messageHandler);
		Bundle data = new Bundle();
		data.putCharSequence("Type", "Toast");
		data.putCharSequence("Data", msg);
		message.setData(data);
		message.sendToTarget();
	}
	
	@Override
	public void popupmenu(Point2df p){
		Rect bounds = new Rect();
		Paint paint = new Paint();
		paint.setTextSize(FONT_SIZE);
		
		int maxWidth = 0;
		for(int i=0; i<PopupMenu.MENU_NUM; i++){
			paint.getTextBounds(PopupMenuEvent.menus[i], 0, PopupMenuEvent.menus[i].length(), bounds);
			if(maxWidth < bounds.width())
				maxWidth = bounds.width();
		}
		
		int width = maxWidth + 2 * PopupMenu.MARGIN;
		int height = (bounds.height()+PopupMenu.MARGIN*2)*PopupMenu.MENU_NUM;
		int frameWidth = getWidth();
		int frameHeight = getHeight();
		
		popupMenu = new PopupMenu(p, projectToMap(new Point2df(0.0f, 0.0f)), width, height, frameWidth, frameHeight);
		redraw();
	}
	
	private void closePopupMenu(){
		popupMenu = null;
		if(null!=selectedShapeKnowledge)
			selectedShapeKnowledge.s.color = selectedShapeColor;
		selectedShapeKnowledge = null;
		
		redraw();
	}
	
	private void popupToast(String msg){
		Toast.makeText(activity, msg.subSequence(0, msg.length()), Toast.LENGTH_SHORT).show();
	}
	
	private int shapeColor2AndroidColor(Color color){
		switch(color){
		case BLACK:
			return android.graphics.Color.BLACK;
		case CYAN:
			return android.graphics.Color.CYAN;
		case RED:
			return android.graphics.Color.RED;
		case YELLOW:
			return android.graphics.Color.YELLOW;
		case BLUE:
			return android.graphics.Color.BLUE;
		case GRAY:
			return android.graphics.Color.GRAY;
		case DARK_GRAY:
			return android.graphics.Color.DKGRAY;
		case GREEN:
			return android.graphics.Color.GREEN;
		case LIGHT_GRAY:
			return android.graphics.Color.LTGRAY;
		case MAGENTA:
			return android.graphics.Color.MAGENTA;
		case ORANGE:
			return android.graphics.Color.rgb(255, 165, 79);
		case PINK:
			return android.graphics.Color.rgb(255, 172, 183);
		case WHITE:
			return android.graphics.Color.WHITE;
		}
		return -1;
	}
	
	private Canvas getCurrentCanvas(){
		return c;
	}

	// Listener
	@Override
	public boolean onTouch(View v, MotionEvent event) {
		
		SketchApp app = BrainmapRealWorld.sketchApp;
		Point2df point = new Point2df(event.getX(), event.getY());
		boolean isMultiTouch = false;
		
		if(event.getPointerCount() > 1)
			isMultiTouch = true;
		
		switch(event.getActionMasked()){
			case MotionEvent.ACTION_DOWN:
				if(null != popupMenu){
					String menu = popupMenu.selectMenu(projectToMap(point));
					if(null == menu)
						closePopupMenu();
				}
				
				else if(isMultiTouch)
					app.postEvent(new PressEvent(point, PressEvent.ACTION.PRESS, PressEvent.TYPE.RIGHT));
				else
					app.postEvent(new PressEvent(point, PressEvent.ACTION.PRESS, PressEvent.TYPE.LEFT));
				
				break;
				
			case MotionEvent.ACTION_UP:
				if(null != popupMenu){
					String menu = popupMenu.selectMenu(projectToMap(point));
					
					if(null != menu){
						if(menu.equals("Edit Color")){
							popupMenu.showPalette();
							redraw();
						}else{
							app.postEvent(new PopupMenuEvent(menu));
							closePopupMenu();
						}
					}
					//System.out.println("AndroidGuiAdapter::onTouch:Seleceted Menu : "+menu);
				}
				
				else if(isMultiTouch)
					app.postEvent(new PressEvent(point, PressEvent.ACTION.RELEASE, PressEvent.TYPE.RIGHT));
				else
					app.postEvent(new PressEvent(point, PressEvent.ACTION.RELEASE, PressEvent.TYPE.LEFT));
				
				break;
				
			case MotionEvent.ACTION_MOVE:
				if(null != popupMenu){}
				
				else if(isMultiTouch)
					app.postEvent(new PressEvent(point, PressEvent.ACTION.DRAG, PressEvent.TYPE.RIGHT));
				else
					app.postEvent(new PressEvent(point, PressEvent.ACTION.DRAG, PressEvent.TYPE.LEFT));
				
				break;
		}
		return true;
	}
}


