/*

   Copyright 2010, 2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package com.sri.brainmap;

import cf.app.brainmap.*;
 
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;  
import java.io.InputStream;
import java.util.Map;

import android.os.Bundle;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Canvas;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.support.v4.app.NavUtils;

@SuppressLint("NewApi")
public class BrainmapActivity extends Activity {
	
	// Home directory for android
	private static final String HOME = "/sdcard/brainmap";
	
	// fileimage in assets
	private static final String FILE_IMAGE_NAME = "file.png";
	
	// Activity Request Code for starting FileBrowser activity
	public static final int FILEBROWSER_ACTIVITY_REQUEST_CODE = 0;

	// Android GUI Adapter
	private AndroidGuiAdapter gui;
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initialize();
    }
	
	@Override
	public void onDestroy(){
		super.onDestroy();
		finish();
	}
	
	public void initialize(){
		gui = new AndroidGuiAdapter(this);
		
		new Thread(){
			@Override
			public void run(){
				try{
					FileAdapter fa = new FileAdapter();
					InputStream is = getResources().getAssets().open("file.png");
					fa.copyFile(is, fa.getDefaultFileImagePath());
					
					new BrainmapRealEngine(gui, fa).run();
				}catch(Exception e){e.printStackTrace();}
			}
		}.start();
	}
		
	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		//System.out.println("BrainmapActivity::dispatchKetEvent:KeyEvent "+event);
		gui.onKey(gui, event.getKeyCode(), event);
	    return super.dispatchKeyEvent(event);
	}
    
    public void onActivityResult(int requestCode, int resultCode, Intent intent){
    	super.onActivityResult(requestCode, resultCode, intent);
    	
    	switch(requestCode){
    	case FILEBROWSER_ACTIVITY_REQUEST_CODE:
    		if(RESULT_OK == resultCode)
    			gui.onFilebrowserActivityResult(intent);
    		break;
    	}
    }
	
	public void onBackPressed(){
		//Log.i(BrainmapActivity.TAG, "BACK!!!");	
	}   
}
