/*

   Copyright 2010, 2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package com.sri.brainmap;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;

import cf.app.brainmap.BrainmapRealEngine;
import cf.app.brainmap.Session;

import com.sri.brainmap.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class SessionBrowserView extends LinearLayout implements OnItemClickListener, OnClickListener{
	
	// Session Browser Results
	public static enum RESULT {LOAD_SESSION, CREATE_SESSION, JOIN_SESSION};
	
	// Views
	ListView lv;
	Button loadButton;
	Button createButton;
	
	// Sessions for ListView
	ArrayList<Session> sessions = null;
	
	// For activity control
	Activity activity;
	AndroidGuiAdapter gui;
	
	// For updating GUI
	Handler messageHandler;	
	
	public SessionBrowserView(Activity activity, AndroidGuiAdapter gui){
		super(activity);
		this.gui = gui;
		this.activity = activity;
		
		// Init gui
		initialize();
		refreshList(null);
	}
	
	private void initialize(){
		// Inflate Layout
		LayoutInflater.from(activity).inflate(R.layout.sessionbrowser, this, true);
		
		lv = (ListView)findViewById(R.id.sessionlistView);
		loadButton = (Button)findViewById(R.id.loadbutton);
		createButton = (Button)findViewById(R.id.createbutton);
		
		// Add listeners
		lv.setOnItemClickListener(this);
		loadButton.setOnClickListener(this);
		createButton.setOnClickListener(this);
		
		messageHandler = generateMsgHandler();
	}
	
	public void refreshList(ArrayList<Session> sessions){
		this.sessions = sessions;
		if(null != sessions){
			Message msg = Message.obtain(messageHandler);
			Bundle data = new Bundle();
			data.putCharSequence("Type", "Invalidate");
			msg.setData(data);
			msg.sendToTarget();
		}
	}
	
	/*	Type(String)	| Data
	 *-------------------------
	 *  Visible			|
	 *  Invisible		|
	 *  Invalidate		| ArrayList<Session>
	*/
	private Handler generateMsgHandler(){
		
		return new Handler(){
			public void handleMessage(Message msg){				
				Bundle b = msg.getData();
				String type = b.getCharSequence("Type").toString();
				
				if(type.equals("Invalidate"))
					invalidate(sessions);
				
				else if(type.equals("Visible"))
					setSessionBrowserVisible(true);
					
				else if(type.equals("Invisible"))
					setSessionBrowserVisible(false);
			}
		};
	}
	
	private void invalidate(ArrayList<Session> sessions){
		lv.setAdapter(new SessionArrayAdapter(getContext(), sessions));
	}
	
	private void setSessionBrowserVisible(boolean flag){
		if(flag)
			activity.setContentView(this);
		else
			setVisibility(LinearLayout.GONE);
	}
	
	@Override
	public void onClick(View v) {
		Button button = (Button)v;
		
		if(button == loadButton){
			gui.onSessionbrowserResult(RESULT.LOAD_SESSION, null);
			setVisibility(LinearLayout.GONE);
		}
		
		else if(button == createButton)
			createSession();		
	}
	
	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position, long id){
		Session session = sessions.get(position);
		
		gui.onSessionbrowserResult(RESULT.JOIN_SESSION, session);
		
		setVisibility(LinearLayout.GONE);
	}
	
	private void createSession(){
		new SessionCreateDialog(getContext()).show();
	}
	
	class SessionCreateDialog extends AlertDialog.Builder{
		private boolean isCreateButtonPressed = false;
		
		public SessionCreateDialog(Context c){
			super(c);
			final EditText textfield = new EditText(c);
			
			setView(textfield);
			setMessage("Input Title");
			setPositiveButton("Create", new DialogInterface.OnClickListener(){
				@Override
				public void onClick(DialogInterface dialog, int which) {
					if(isCreateButtonPressed)
						return;
					
					String title = textfield.getText().toString();
					gui.onSessionbrowserResult(RESULT.CREATE_SESSION, new Session(title, BrainmapRealEngine.myEngineId()));
					setVisibility(LinearLayout.GONE);
					isCreateButtonPressed = true;
				}
			});
			setNegativeButton("Cancel", new DialogInterface.OnClickListener(){
				@Override
				public void onClick(DialogInterface dialog, int which) {
				}
			});
		}
	} 
}

class SessionArrayAdapter extends ArrayAdapter<Session>{
	
	// Id of Session Browser Layout
	private final static int RESOURCE_ID = R.layout.sessionrow;	
	
	private Context context;
	private ArrayList<Session> sessions;
	
	public SessionArrayAdapter(Context context, ArrayList<Session> sessions) {
		super(context, RESOURCE_ID, sessions);
		this.context = context;
		this.sessions = sessions;
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent){
		LayoutInflater inflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View rowView = inflater.inflate(RESOURCE_ID, parent, false);
		
		TextView titletv = (TextView)rowView.findViewById(R.id.titletv);
		TextView membertv = (TextView)rowView.findViewById(R.id.membertv);
		
		Session session = sessions.get(position);
		
		titletv.setText(session.title);
		membertv.setText("");
		
		return rowView;
	}
}
