/*

   Copyright 2012-2013 SRI International

   Licensed under the Apache License, Version 2.0 (the "License"); you may
   not use this file except in compliance with the License. You may obtain
   a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

   The development was supported by National Science Foundation Grant 0932397:
   A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.brainmap.jre;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.table.*;

import cf.app.brainmap.*;
import cf.app.brainmap.events.*;
import cf.common.Engine;

public class SessionBrowser extends JFrame implements MouseListener, ActionListener{
	
	// Print session's title only
	private static final boolean TITLE_ONLY = true;
	
	// Double click delay
	private static final int CLICK_DELAY_FOR_DOUBLE_CLICK = 500; //500ms
	
	// Sizes for GUI components
	private static final int ROW_HEIGHT = 25;
	private static final int TITLE_WIDTH = 300;
	private static final int MEMBER_NUM_WIDTH = 80;
	private static final int SESSION_BROWSER_WIDTH = TITLE_WIDTH+MEMBER_NUM_WIDTH+100;
	private static final int BUTTON_BAR_WIDTH = SESSION_BROWSER_WIDTH;
	private static final int BUTTON_BAR_HEIGHT = 30;
	private static final int SCROLLPANE_WIDTH = SESSION_BROWSER_WIDTH;
	private static final int SCROLLPANE_HEIGHT = 500;
	private static final int SESSION_BROWSER_HEIGHT = BUTTON_BAR_HEIGHT+SCROLLPANE_HEIGHT+50;
	
	private JREGuiAdapter gui;
	
	private JTable table;
	String[] columnNames = {"Title"};
	
	JPanel mainPanel;
	SpringLayout.Constraints preCons = null;
	
	int clickedIndex = -1;
	private long clickedTime;
	private ArrayList<Session> sessions = new ArrayList<Session>();
	
	DefaultTableCellRenderer centerRenderer;
	
	public SessionBrowser(JREGuiAdapter gui, Point p){
		this.gui = gui;
		
		initFrame();
		initMainPanel();
		addButtonBar();
		addTable();
		setSessions(sessions);
		
		setVisible(true, p);
	}
	
	public void setVisible(boolean visible, int x, int y){
		setLocation(x, y);
		super.setVisible(true);
	}
	
	public void setVisible(boolean visible, Point p){
		setVisible(visible, p.x, p.y);
	}
	
	private boolean containsSessionList(ArrayList<Session> list1, ArrayList<Session> list2){
		if(null==list1 && null==list2)
			return true;
		
		else if(null==list2)
			return true;
		
		else if(null==list1)
			return false;
		
		for(Session s2: list2){
			int i=0;
			for(; i<list1.size(); i++){
				Session s1 = list1.get(i);
				if(s1.equals(s2))
					break;
			}
			if(i == list1.size())
				return false;
		}
		
		return true;
	}
	
	private void initFrame(){
		setTitle("Brainstorming Sessions");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(SESSION_BROWSER_WIDTH, SESSION_BROWSER_HEIGHT);
		int browser_x = (Toolkit.getDefaultToolkit().getScreenSize().width - SESSION_BROWSER_WIDTH) / 2;
		int browser_y = (Toolkit.getDefaultToolkit().getScreenSize().height - SESSION_BROWSER_HEIGHT) / 2;
		setLocation(browser_x, browser_y);
	}
	
	private void initMainPanel(){
		mainPanel = new JPanel();
		add(mainPanel);
	}

	private void addButtonBar(){
		JButton loadButton = new JButton("Load");
		JButton createButton = new JButton("Create");
		
		loadButton.addActionListener(this);
		createButton.addActionListener(this);
		
		JPanel panel = new JPanel();
		
		panel.setPreferredSize(new Dimension(BUTTON_BAR_WIDTH, BUTTON_BAR_HEIGHT));
		panel.add(loadButton, BorderLayout.WEST);
		panel.add(createButton, BorderLayout.EAST);
		
		mainPanel.add(panel, BorderLayout.NORTH);
	}
	
	private void addTable(){
		if(TITLE_ONLY)
			table = new JTable(0,1);
		else
			table = new JTable(0,2);
		
		table.setShowHorizontalLines(true);
		table.setShowVerticalLines(true);
		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);
		table.setRowHeight(ROW_HEIGHT);
		table.addMouseListener(this);
		
		centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment( JLabel.CENTER );
		
		table.getColumnModel().getColumn(0).setPreferredWidth(TITLE_WIDTH);
		
		if(!TITLE_ONLY){
			table.getColumnModel().getColumn(1).setPreferredWidth(MEMBER_NUM_WIDTH);
			table.getColumnModel().getColumn(1).setCellRenderer( centerRenderer );
		}
		
		try{
			JScrollPane scrollPane = new JScrollPane(table);
			scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			scrollPane.setPreferredSize(new Dimension(SCROLLPANE_WIDTH, SCROLLPANE_HEIGHT));
			
			mainPanel.add(scrollPane, BorderLayout.CENTER);
		}catch(Exception e){}
	}
	
	public void updateSessions(ArrayList<Session> sessions){
		if(containsSessionList(this.sessions, sessions) && containsSessionList(sessions, this.sessions))
			return;
		
		for(int i=this.sessions.size()-1; i>=0; i--)
			removeSession(i);
		
		for(Session s : sessions)
			addSession(s);
		
		if(-1 != clickedIndex)
			table.setRowSelectionInterval(clickedIndex, clickedIndex);
	}
		
	private void addSession(Session s){
		String[] row ;
		
		if(TITLE_ONLY)
			row = new String[1];
		else{
			row = new String[2];
			row[1] = Integer.toString(s.engines.size());
		}
		
		row[0] = s.title;
		
		((DefaultTableModel)table.getModel()).addRow(row);
		this.sessions.add(s);
	}
	
	private void removeSession(int index){
		((DefaultTableModel)table.getModel()).removeRow(index);
		this.sessions.remove(index);
	}
	
	private void setSessions(ArrayList<Session> sessions){
		DefaultTableModel model;
		
		if(null == sessions){
			model = new DefaultTableModel(columnNames, 0){
				@Override
				public boolean isCellEditable(int row, int column){
					return false;
				}
			};
		}
		
		else{
			this.sessions = new ArrayList<Session>();
			String[][] data;
			
			if(TITLE_ONLY)
				data = new String[sessions.size()][1];
			else
				data = new String[sessions.size()][2];
			
			for(int i=0; i<sessions.size(); i++){
				Session s = sessions.get(i);
				this.sessions.add(s);
				data[i][0] = s.title;
				
				if(!TITLE_ONLY)
					data[i][1] = Integer.toString(s.engines.size());
			}
			
			model = new DefaultTableModel(data, columnNames){
				@Override
				public boolean isCellEditable(int row, int column){
					return false;
				}
			};
		}
		
		table.setModel(model);
		
		table.getColumnModel().getColumn(0).setPreferredWidth(TITLE_WIDTH);
		
		if(!TITLE_ONLY){
			table.getColumnModel().getColumn(1).setPreferredWidth(MEMBER_NUM_WIDTH);
			table.getColumnModel().getColumn(1).setCellRenderer( centerRenderer );
		}
		
		if(-1 != clickedIndex)
			table.setRowSelectionInterval(clickedIndex, clickedIndex);
	}
	
	
	private void doubleClicked(Session s){
		SketchApp app = BrainmapRealWorld.sketchApp;
		app.postEvent(new SessionEvent(SessionEvent.TYPE.JOIN, s, false, 0.0, BrainmapRealWorld.brainmapApp));
	}
	
	private void loadSession(){
		gui.popupLoadDialog();
		setVisible(false);
	}
	
	private void createSession(){
		SketchApp app = BrainmapRealWorld.sketchApp;
		
		String title = JOptionPane.showInputDialog(this, "Input Session Title");
		
		Session s = new Session(title, BrainmapRealEngine.myEngineId());
		
		app.postEvent(new SessionEvent(SessionEvent.TYPE.CREATE, s, false, 0.0, BrainmapRealWorld.brainmapApp));
		setVisible(false);
	}
	
	@Override
	public void actionPerformed(ActionEvent arg0) {
		String buttonName = ((JButton)arg0.getSource()).getText();
		
		if(buttonName.equals("Load"))
			loadSession();
		
		else if(buttonName.equals("Create"))
			createSession();
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
		if(-1 == clickedIndex){
			clickedTime = System.currentTimeMillis();
			clickedIndex = table.getSelectedRow();
			
		}else{
			if(table.getSelectedRow() == clickedIndex && (System.currentTimeMillis() - clickedTime) < CLICK_DELAY_FOR_DOUBLE_CLICK){
				//System.out.println("Selected!! " + clickedIndex);
				doubleClicked(sessions.get(clickedIndex));
				clickedIndex = -1;
			}
			
			else{
				clickedIndex = table.getSelectedRow();
				clickedTime = System.currentTimeMillis();
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mousePressed(MouseEvent arg0) {}
	public void mouseReleased(MouseEvent arg0) {}
	
	private static class HeaderRenderer implements TableCellRenderer{
		DefaultTableCellRenderer renderer;
		
		public HeaderRenderer(JTable table){
			renderer = (DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer();
		}
		
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int col){
			Component c = renderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, col);
			JLabel label = (JLabel)c;
			label.setHorizontalAlignment(JLabel.CENTER);
			return label;
		}
	}
}

